@@ -1,19 +1,76 @@
 """
-Image compression utils.
-"""
-from __future__ import annotations
+Image compression utilities for PSD channel data.
 
-from typing import Iterator
+This subpackage provides compression and decompression codecs for raw pixel
+data in PSD files. Adobe Photoshop supports multiple compression methods for
+channel data to reduce file size.
+
+Supported compression methods:
+
+- **RAW** (``Compression.RAW``): Uncompressed raw pixel data
+- **RLE** (``Compression.RLE``): Apple PackBits run-length encoding
+- **ZIP** (``Compression.ZIP``): ZIP/Deflate compression without prediction
+- **ZIP_WITH_PREDICTION** (``Compression.ZIP_WITH_PREDICTION``): ZIP with delta encoding
+
+The RLE codec includes both a pure Python implementation and a Cython-optimized
+version (``_rle.pyx``) that provides significant performance improvements. The
+Cython version is used automatically when available, with graceful fallback to
+pure Python.
+
+Key functions:
+
+- :py:func:`compress`: Compress raw pixel data using specified method
+- :py:func:`decompress`: Decompress pixel data back to raw bytes
+- :py:func:`encode_rle`: RLE encoding for a single channel
+- :py:func:`decode_rle`: RLE decoding for a single channel
+
+Example usage::
+
+    from psd_tools.compression import compress, decompress
+    from psd_tools.constants import Compression
+
+    # Compress raw channel data
+    compressed = compress(
+        data=raw_pixels,
+        compression=Compression.RLE,
+        width=100,
+        height=100,
+        depth=8,
+        version=1
+    )
+
+    # Decompress back to raw data
+    raw_pixels = decompress(
+        data=compressed,
+        compression=Compression.RLE,
+        width=100,
+        height=100,
+        depth=8,
+        version=1
+    )
+
+Performance notes:
+
+- RLE is most effective for images with large uniform areas
+- ZIP with prediction works well for continuous-tone images
+- The Cython RLE codec can be 10-100x faster than pure Python
+- Compression method is chosen per-channel when saving PSD files
+
+The compression module handles various bit depths (8, 16, 32-bit per channel)
+and implements delta encoding for improved compression ratios on certain
+image types.
+"""
 
 import array
 import io
 import logging
 import zlib
+from typing import Iterator, Union
 
 from PIL import Image
 
 from psd_tools.constants import Compression
-from psd_tools.utils import (
+from psd_tools.psd.bin_utils import (
     be_array_from_bytes,
     be_array_to_bytes,
     read_be_array,
@@ -21,7 +78,7 @@ from psd_tools.utils import (
 )
 
 try:
-    from . import _rle as rle_impl
+    from . import _rle as rle_impl  # type: ignore[import-not-found,attr-defined]
 except ImportError:
     from . import rle as rle_impl
 
@@ -80,7 +137,7 @@ def decompress(
     """
     length = width * height * max(1, depth // 8)
 
-    result = None
+    result: bytes | None = None
     if compression == Compression.RAW:
         result = data[:length]
     elif compression == Compression.RLE:
@@ -99,6 +156,7 @@ def decompress(
         else:
             assert len(result) == length, "len=%d, expected=%d" % (len(result), length)
 
+    assert result is not None
     return result
 
 
@@ -134,7 +192,9 @@ def decode_rle(data: bytes, width: int, height: int, depth: int, version: int) -
         raise
 
 
-def encode_prediction(data: bytes | bytearray, w: int, h: int, depth: int) -> bytes:
+def encode_prediction(
+    data: Union[bytes, bytearray], w: int, h: int, depth: int
+) -> bytes:
     if depth == 8:
         arr = array.array("B", data)
         arr = _delta_encode(arr, 0x100, w, h)
@@ -147,12 +207,12 @@ def encode_prediction(data: bytes | bytearray, w: int, h: int, depth: int) -> by
         arr = array.array("B", data)
         arr = _shuffle_byte_order(arr, w, h)
         arr = _delta_encode(arr, 0x100, w * 4, h)
-        return getattr(arr, "tobytes", getattr(arr, "tostring", None))()
+        return arr.tobytes()
     else:
         raise ValueError("Invalid pixel size %d" % (depth))
 
 
-def decode_prediction(data: bytes, w: int, h: int, depth: int) -> array.array:
+def decode_prediction(data: bytes, w: int, h: int, depth: int) -> bytes:
     if depth == 8:
         arr = be_array_from_bytes("B", data)
         arr = _delta_decode(arr, 0x100, w, h)
@@ -166,7 +226,7 @@ def decode_prediction(data: bytes, w: int, h: int, depth: int) -> array.array:
     else:
         raise ValueError("Invalid pixel size %d" % (depth))
 
-    return getattr(arr, "tobytes", getattr(arr, "tostring", None))()
+    return arr.tobytes()
 
 
 def _delta_encode(arr: array.array, mod: int, w: int, h: int) -> array.array:
