@@ -16,11 +16,10 @@ Pretty printing is the best approach to check the descriptor content::
     pprint(descriptor)
 """
 
-from __future__ import absolute_import, unicode_literals
-
 import logging
+from typing import Any, BinaryIO, Dict, Iterator, TypeVar
 
-import attr
+from attrs import define, field
 
 from psd_tools.constants import OSType
 from psd_tools.psd.base import (
@@ -33,8 +32,7 @@ from psd_tools.psd.base import (
     StringElement,
 )
 from psd_tools.terminology import Enum, Event, Form, Key, Klass, Type, Unit
-from psd_tools.utils import (
-    new_registry,
+from psd_tools.psd.bin_utils import (
     read_fmt,
     read_length_block,
     read_unicode_string,
@@ -45,18 +43,24 @@ from psd_tools.utils import (
     write_padding,
     write_unicode_string,
 )
+from psd_tools.registry import new_registry
 from psd_tools.validators import in_
 
 logger = logging.getLogger(__name__)
 
 TYPES, register = new_registry(attribute="ostype")
 
+T = TypeVar("T")
+
 _TERMS = set(
-    item.value for kls in (Klass, Enum, Event, Form, Key, Type, Unit) for item in kls if len(item.value) == 4
+    item.value
+    for kls in (Klass, Enum, Event, Form, Key, Type, Unit)
+    for item in kls
+    if len(item.value) == 4
 )
 
 
-def read_length_and_key(fp):
+def read_length_and_key(fp: BinaryIO) -> bytes:
     """
     Helper to read descriptor key.
     """
@@ -68,7 +72,7 @@ def read_length_and_key(fp):
     return key
 
 
-def write_length_and_key(fp, value):
+def write_length_and_key(fp: BinaryIO, value: bytes) -> int:
     """
     Helper to write descriptor key.
     """
@@ -78,8 +82,11 @@ def write_length_and_key(fp, value):
 
 
 class _DescriptorMixin(DictElement):
+    name: str = ""
+    classID: bytes = b"\x00\x00\x00\x00"
+
     @classmethod
-    def _read_body(cls, fp):
+    def _read_body(cls, fp: BinaryIO) -> Dict[str, Any]:
         name = read_unicode_string(fp, padding=1)
         classID = read_length_and_key(fp)
         items = []
@@ -88,12 +95,12 @@ class _DescriptorMixin(DictElement):
             key = read_length_and_key(fp)
             ostype = OSType(fp.read(4))
             kls = TYPES.get(ostype)
-            value = kls.read(fp)
+            value = kls.read(fp)  # type: ignore[union-attr]
             items.append((key, value))
 
         return dict(name=name, classID=classID, items=items)
 
-    def _write_body(self, fp):
+    def _write_body(self, fp: BinaryIO) -> int:
         written = write_unicode_string(fp, self.name, padding=1)
         written += write_length_and_key(fp, self.classID)
         written += write_fmt(fp, "I", len(self))
@@ -104,14 +111,15 @@ class _DescriptorMixin(DictElement):
         return written
 
     @classmethod
-    def _key_converter(cls, key):
+    def _key_converter(cls, key: Any) -> bytes:
         if hasattr(key, "encode"):
             return key.encode("ascii")
         return getattr(key, "value", key)
 
-    def _repr_pretty_(self, p, cycle):
+    def _repr_pretty_(self, p: Any, cycle: bool) -> None:
         if cycle:
-            return "(...)"
+            p.text("(...)")
+            return
 
         prefix = "{cls}({name}){{".format(
             cls=self.__class__.__name__,
@@ -134,7 +142,7 @@ class _DescriptorMixin(DictElement):
 
 
 @register(OSType.DESCRIPTOR)
-@attr.s(repr=False)
+@define(repr=False)
 class Descriptor(_DescriptorMixin):
     """
     Dict-like descriptor structure.
@@ -161,19 +169,19 @@ class Descriptor(_DescriptorMixin):
         bytes in :py:class:`~psd_tools.terminology.Klass`
     """
 
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=Klass.Null.value)
+    name: str = ""
+    classID: bytes = Klass.Null.value
 
     @classmethod
-    def read(cls, fp):
-        return cls(**cls._read_body(fp))
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
+        return cls(**cls._read_body(fp))  # type: ignore[attr-defined]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return self._write_body(fp)
 
 
 @register(OSType.OBJECT_ARRAY)
-@attr.s(repr=False)
+@define(repr=False)
 class ObjectArray(_DescriptorMixin):
     """
     Object array structure almost equivalent to
@@ -192,23 +200,23 @@ class ObjectArray(_DescriptorMixin):
         bytes in :py:class:`~psd_tools.terminology.Klass`
     """
 
-    items_count = attr.ib(default=0, type=int)
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=Klass.Null.value)
+    items_count: int = 0
+    name: str = ""
+    classID: bytes = Klass.Null.value
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         items_count = read_fmt("I", fp)[0]
-        return cls(items_count=items_count, **cls._read_body(fp))
+        return cls(items_count=items_count, **cls._read_body(fp))  # type: ignore[attr-defined,call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_fmt(fp, "I", self.items_count)
         written += self._write_body(fp)
         return written
 
 
 @register(OSType.LIST)
-@attr.s(repr=False)
+@define(repr=False)
 class List(ListElement):
     """
     List structure.
@@ -220,17 +228,17 @@ class List(ListElement):
     """
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         items = []
         count = read_fmt("I", fp)[0]
         for _ in range(count):
             key = OSType(fp.read(4))
             kls = TYPES.get(key)
-            value = kls.read(fp)
+            value = kls.read(fp)  # type: ignore[union-attr]
             items.append(value)
-        return cls(items)
+        return cls(items)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_fmt(fp, "I", len(self))
         for item in self:
             written += write_bytes(fp, item.ostype.value)
@@ -239,7 +247,7 @@ class List(ListElement):
 
 
 @register(OSType.PROPERTY)
-@attr.s(repr=False)
+@define(repr=False)
 class Property(BaseElement):
     """
     Property structure.
@@ -257,18 +265,18 @@ class Property(BaseElement):
         bytes in :py:class:`~psd_tools.terminology.Key`
     """
 
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
-    keyID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
+    name: str = ""
+    classID: bytes = b"\x00\x00\x00\x00"
+    keyID: bytes = b"\x00\x00\x00\x00"
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         name = read_unicode_string(fp)
         classID = read_length_and_key(fp)
         keyID = read_length_and_key(fp)
-        return cls(name, classID, keyID)
+        return cls(name, classID, keyID)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_unicode_string(fp, self.name)
         written += write_length_and_key(fp, self.classID)
         written += write_length_and_key(fp, self.keyID)
@@ -276,7 +284,7 @@ class Property(BaseElement):
 
 
 @register(OSType.UNIT_FLOAT)
-@attr.s(slots=True, repr=False, eq=False, order=False)
+@define(repr=False, eq=False, order=False)
 class UnitFloat(NumericElement):
     """
     Unit float structure.
@@ -290,32 +298,33 @@ class UnitFloat(NumericElement):
         `float` value
     """
 
-    value = attr.ib(default=0.0, type=float)
-    unit = attr.ib(default=Unit._None)
+    value: float = 0.0
+    unit: Unit = Unit._None
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         unit, value = read_fmt("4sd", fp)
         try:
             unit = Unit(unit)
         except ValueError:
             logger.warning("Using Enum for Unit field")
             unit = Enum(unit)
-        return cls(unit=unit, value=value)
+        return cls(unit=unit, value=value)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return write_fmt(fp, "4sd", self.unit.value, self.value)
 
-    def _repr_pretty_(self, p, cycle):
+    def _repr_pretty_(self, p: Any, cycle: bool) -> None:
         if cycle:
-            return self.__repr__()
+            p.text(self.__repr__())
+            return
         p.pretty(self.value)
         p.text(" ")
         p.text(self.unit.name)
 
 
 @register(OSType.UNIT_FLOATS)
-@attr.s(repr=False)
+@define(repr=False)
 class UnitFloats(BaseElement):
     """
     Unit floats structure.
@@ -329,11 +338,11 @@ class UnitFloats(BaseElement):
         List of `float` values
     """
 
-    unit = attr.ib(default=Unit._None)
-    values = attr.ib(factory=list)
+    unit: Unit = Unit._None
+    values: list = field(factory=list)
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         unit, count = read_fmt("4sI", fp)
         try:
             unit = Unit(unit)
@@ -341,9 +350,9 @@ class UnitFloats(BaseElement):
             logger.warning("Using Enum for Unit field")
             unit = Enum(unit)
         values = list(read_fmt("%dd" % count, fp))
-        return cls(unit=unit, values=values)
+        return cls(unit=unit, values=values)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return write_fmt(
             fp,
             "4sI%dd" % len(self.values),
@@ -352,14 +361,14 @@ class UnitFloats(BaseElement):
             *self.values,
         )
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[float]:
         for value in self.values:
             yield value
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> float:
         return self.values[index]
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.values)
 
 
@@ -374,14 +383,14 @@ class Double(NumericElement):
     """
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         return cls(*read_fmt("d", fp))
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return write_fmt(fp, "d", self.value)
 
 
-@attr.s(repr=False)
+@define(repr=False)
 class Class(BaseElement):
     """
     Class structure.
@@ -395,16 +404,16 @@ class Class(BaseElement):
         bytes in :py:class:`~psd_tools.terminology.Klass`
     """
 
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
+    name: str = ""
+    classID: bytes = b"\x00\x00\x00\x00"
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         name = read_unicode_string(fp)
         classID = read_length_and_key(fp)
-        return cls(name, classID)
+        return cls(name, classID)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_unicode_string(fp, self.name)
         written += write_length_and_key(fp, self.classID)
         return written
@@ -424,7 +433,7 @@ class String(StringElement):
 
 
 @register(OSType.ENUMERATED_REFERENCE)
-@attr.s(repr=False)
+@define(repr=False)
 class EnumeratedReference(BaseElement):
     """
     Enumerated reference structure.
@@ -446,20 +455,20 @@ class EnumeratedReference(BaseElement):
         bytes in :py:class:`~psd_tools.terminology.Enum`
     """
 
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
-    typeID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
-    enum = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
+    name: str = ""
+    classID: bytes = b"\x00\x00\x00\x00"
+    typeID: bytes = b"\x00\x00\x00\x00"
+    enum: bytes = b"\x00\x00\x00\x00"
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         name = read_unicode_string(fp)
         classID = read_length_and_key(fp)
         typeID = read_length_and_key(fp)
         enum = read_length_and_key(fp)
-        return cls(name, classID, typeID, enum)
+        return cls(name, classID, typeID, enum)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_unicode_string(fp, self.name)
         written += write_length_and_key(fp, self.classID)
         written += write_length_and_key(fp, self.typeID)
@@ -468,7 +477,7 @@ class EnumeratedReference(BaseElement):
 
 
 @register(OSType.OFFSET)
-@attr.s(repr=False)
+@define(repr=False)
 class Offset(BaseElement):
     """
     Offset structure.
@@ -486,18 +495,18 @@ class Offset(BaseElement):
         `int` value
     """
 
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
-    value = attr.ib(default=0)
+    name: str = ""
+    classID: bytes = b"\x00\x00\x00\x00"
+    value: int = 0
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         name = read_unicode_string(fp)
         classID = read_length_and_key(fp)
         offset = read_fmt("I", fp)[0]
-        return cls(name, classID, offset)
+        return cls(name, classID, offset)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_unicode_string(fp, self.name)
         written += write_length_and_key(fp, self.classID)
         written += write_fmt(fp, "I", self.value)
@@ -515,10 +524,10 @@ class Bool(BooleanElement):
     """
 
     @classmethod
-    def read(cls, fp):
-        return cls(read_fmt("?", fp)[0])
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
+        return cls(read_fmt("?", fp)[0])  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return write_fmt(fp, "?", self.value)
 
 
@@ -533,10 +542,10 @@ class LargeInteger(IntegerElement):
     """
 
     @classmethod
-    def read(cls, fp):
-        return cls(read_fmt("q", fp)[0])
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
+        return cls(read_fmt("q", fp)[0])  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return write_fmt(fp, "q", self.value)
 
 
@@ -551,15 +560,15 @@ class Integer(IntegerElement):
     """
 
     @classmethod
-    def read(cls, fp):
-        return cls(read_fmt("i", fp)[0])
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
+        return cls(read_fmt("i", fp)[0])  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         return write_fmt(fp, "i", self.value)
 
 
 @register(OSType.ENUMERATED)
-@attr.s(repr=False)
+@define(repr=False)
 class Enumerated(BaseElement):
     """
     Enum structure.
@@ -573,30 +582,31 @@ class Enumerated(BaseElement):
         bytes in :py:class:`~psd_tools.terminology.Enum`
     """
 
-    typeID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
-    enum = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
+    typeID: bytes = b"\x00\x00\x00\x00"
+    enum: bytes = b"\x00\x00\x00\x00"
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         typeID = read_length_and_key(fp)
         enum = read_length_and_key(fp)
-        return cls(typeID, enum)
+        return cls(typeID, enum)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_length_and_key(fp, self.typeID)
         written += write_length_and_key(fp, self.enum)
         return written
 
-    def _repr_pretty_(self, p, cycle):
+    def _repr_pretty_(self, p: Any, cycle: bool) -> None:
         if cycle:
-            return self.__repr__()
+            p.text(self.__repr__())
+            return
         p.text("(")
         p.pretty(getattr(self.typeID, "name", self.typeID))
         p.text(", ")
         p.pretty(getattr(self.enum, "name", self.enum))
         p.text(")")
 
-    def get_name(self):
+    def get_name(self) -> str:
         """Get enum name."""
         if len(self.enum) == 4:
             try:
@@ -607,7 +617,7 @@ class Enumerated(BaseElement):
 
 
 @register(OSType.RAW_DATA)
-@attr.s(repr=False)
+@define(repr=False)
 class RawData(BaseElement):
     """
     RawData structure.
@@ -617,14 +627,14 @@ class RawData(BaseElement):
         `bytes` value
     """
 
-    value = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
+    value: bytes = b"\x00\x00\x00\x00"
 
     @classmethod
-    def read(cls, fp):
-        return cls(read_length_block(fp))
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
+        return cls(read_length_block(fp))  # type: ignore[call-arg]
 
-    def write(self, fp):
-        def writer(f):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
+        def writer(f: BinaryIO) -> int:
             if hasattr(self.value, "write"):
                 return self.value.write(f)
             return write_bytes(f, self.value)
@@ -722,7 +732,7 @@ class Index(Integer):
 
 
 @register(OSType.NAME)
-@attr.s(repr=False)
+@define(repr=False)
 class Name(BaseElement):
     """
     Name structure (Undocumented).
@@ -740,25 +750,25 @@ class Name(BaseElement):
         str
     """
 
-    name = attr.ib(default="", type=str)
-    classID = attr.ib(default=b"\x00\x00\x00\x00", type=bytes)
-    value = attr.ib(default="", type=str)
+    name: str = ""
+    classID: bytes = b"\x00\x00\x00\x00"
+    value: str = ""
 
     @classmethod
-    def read(cls, fp):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         name = read_unicode_string(fp)
         classID = read_length_and_key(fp)
         value = read_unicode_string(fp)
-        return cls(name, classID, value)
+        return cls(name, classID, value)  # type: ignore[call-arg]
 
-    def write(self, fp):
+    def write(self, fp: BinaryIO, **kwargs: Any) -> int:
         written = write_unicode_string(fp, self.name)
         written += write_length_and_key(fp, self.classID)
         written += write_unicode_string(fp, self.value)
         return written
 
 
-@attr.s(repr=False)
+@define(repr=False)
 class DescriptorBlock(Descriptor):
     """
     Dict-like Descriptor-based structure that has `version` field. See
@@ -767,21 +777,21 @@ class DescriptorBlock(Descriptor):
     .. py:attribute:: version
     """
 
-    version = attr.ib(default=16, type=int, validator=in_((16,)))
+    version: int = field(default=16, validator=in_((16,)))
 
     @classmethod
-    def read(cls, fp, **kwargs):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         version = read_fmt("I", fp)[0]
-        return cls(version=version, **cls._read_body(fp))
+        return cls(version=version, **cls._read_body(fp))  # type: ignore[attr-defined,call-arg]
 
-    def write(self, fp, padding=4, **kwargs):
+    def write(self, fp: BinaryIO, padding: int = 4, **kwargs: Any) -> int:
         written = write_fmt(fp, "I", self.version)
         written += self._write_body(fp)
         written += write_padding(fp, written, padding)
         return written
 
 
-@attr.s(repr=False)
+@define(repr=False)
 class DescriptorBlock2(Descriptor):
     """
     Dict-like Descriptor-based structure that has `version` and
@@ -792,15 +802,15 @@ class DescriptorBlock2(Descriptor):
     .. py:attribute:: data_version
     """
 
-    version = attr.ib(default=1, type=int)
-    data_version = attr.ib(default=16, type=int, validator=in_((16,)))
+    version: int = 1
+    data_version: int = field(default=16, validator=in_((16,)))
 
     @classmethod
-    def read(cls, fp, **kwargs):
+    def read(cls: type[T], fp: BinaryIO, **kwargs: Any) -> T:
         version, data_version = read_fmt("2I", fp)
-        return cls(version=version, data_version=data_version, **cls._read_body(fp))
+        return cls(version=version, data_version=data_version, **cls._read_body(fp))  # type: ignore[attr-defined,call-arg]
 
-    def write(self, fp, padding=4, **kwargs):
+    def write(self, fp: BinaryIO, padding: int = 4, **kwargs: Any) -> int:
         written = write_fmt(fp, "2I", self.version, self.data_version)
         written += self._write_body(fp)
         written += write_padding(fp, written, padding)
