@@ -1,25 +1,24 @@
-from __future__ import absolute_import, unicode_literals
-
 import logging
+from typing import Iterator
 
 import pytest
 
 from psd_tools.api.psd_image import PSDImage
 from psd_tools.terminology import Enum
+from psd_tools.api import effects
 
 from ..utils import full_name
 
 logger = logging.getLogger(__name__)
 
-LAYER_EFFECTS = PSDImage.open(full_name("layer_effects.psd"))
-
 
 @pytest.fixture
-def fixture():
-    yield LAYER_EFFECTS
+def fixture() -> Iterator[PSDImage]:
+    yield PSDImage.open(full_name("layer_effects.psd"))
 
 
-def test_effects(fixture):
+def test_effects(fixture: PSDImage) -> None:
+    assert isinstance(fixture[0].effects, effects.Effects)
     assert isinstance(fixture[0].effects.scale, float)
     assert fixture[0].effects.enabled is True
     for layer in fixture:
@@ -28,8 +27,9 @@ def test_effects(fixture):
         assert effect.enabled is True
 
 
-def test_bevel(fixture):
+def test_bevel(fixture: PSDImage) -> None:
     effect = fixture[1].effects[0]
+    assert isinstance(effect, effects.BevelEmboss)
     assert not hasattr(effect, "blend_mode")
     assert effect.altitude == 30.0
     assert effect.angle == 90.0
@@ -53,8 +53,9 @@ def test_bevel(fixture):
     assert effect.use_texture is False
 
 
-def test_emboss(fixture):
+def test_emboss(fixture: PSDImage) -> None:
     effect = fixture[2].effects[0]
+    assert isinstance(effect, effects.BevelEmboss)
     assert not hasattr(effect, "blend_mode")
     assert effect.altitude == 30.0
     assert effect.angle == 90.0
@@ -78,8 +79,9 @@ def test_emboss(fixture):
     assert effect.use_texture is False
 
 
-def test_outer_glow(fixture):
+def test_outer_glow(fixture: PSDImage) -> None:
     effect = fixture[3].effects[0]
+    assert isinstance(effect, effects.OuterGlow)
     assert effect.anti_aliased is False
     assert effect.blend_mode == Enum.Screen
     assert effect.choke == 0.0
@@ -95,8 +97,9 @@ def test_outer_glow(fixture):
     assert effect.gradient is None
 
 
-def test_inner_glow(fixture):
+def test_inner_glow(fixture: PSDImage) -> None:
     effect = fixture[4].effects[0]
+    assert isinstance(effect, effects.InnerGlow)
     assert effect.anti_aliased is False
     assert effect.blend_mode == Enum.Screen
     assert effect.choke == 0.0
@@ -112,8 +115,9 @@ def test_inner_glow(fixture):
     assert effect.gradient is None
 
 
-def test_inner_shadow(fixture):
+def test_inner_shadow(fixture: PSDImage) -> None:
     effect = fixture[5].effects[0]
+    assert isinstance(effect, effects.InnerShadow)
     assert effect.angle == 90.0
     assert effect.anti_aliased is False
     assert effect.blend_mode == Enum.Multiply
@@ -127,15 +131,17 @@ def test_inner_shadow(fixture):
     assert effect.use_global_light is True
 
 
-def test_color_overlay(fixture):
+def test_color_overlay(fixture: PSDImage) -> None:
     effect = fixture[6].effects[0]
+    assert isinstance(effect, effects.ColorOverlay)
     assert effect.blend_mode == Enum.Normal
     assert effect.color
     assert effect.opacity == 100.0
 
 
-def test_drop_shadow(fixture):
+def test_drop_shadow(fixture: PSDImage) -> None:
     effect = fixture[7].effects[0]
+    assert isinstance(effect, effects.DropShadow)
     assert effect.angle == 90.0
     assert effect.anti_aliased is False
     assert effect.blend_mode == Enum.Multiply
@@ -150,8 +156,9 @@ def test_drop_shadow(fixture):
     assert effect.use_global_light is True
 
 
-def test_gradient_overlay(fixture):
+def test_gradient_overlay(fixture: PSDImage) -> None:
     effect = fixture[8].effects[0]
+    assert isinstance(effect, effects.GradientOverlay)
     assert effect.aligned is True
     assert effect.angle == 87.0
     assert effect.blend_mode == Enum.Normal
@@ -164,8 +171,9 @@ def test_gradient_overlay(fixture):
     assert effect.type == Enum.Linear
 
 
-def test_pattern_overlay(fixture):
+def test_pattern_overlay(fixture: PSDImage) -> None:
     effect = fixture[9].effects[0]
+    assert isinstance(effect, effects.PatternOverlay)
     assert effect.aligned is True
     assert effect.blend_mode == Enum.Normal
     assert effect.opacity == 100.0
@@ -174,8 +182,9 @@ def test_pattern_overlay(fixture):
     assert effect.scale == 100.0
 
 
-def test_stroke(fixture):
+def test_stroke(fixture: PSDImage) -> None:
     effect = fixture[10].effects[0]
+    assert isinstance(effect, effects.Stroke)
     assert effect.blend_mode == Enum.Normal
     assert effect.fill_type == Enum.SolidColor
     assert effect.opacity == 100.0
@@ -187,8 +196,9 @@ def test_stroke(fixture):
     assert effect.pattern is None
 
 
-def test_satin(fixture):
-    effect = fixture[11].effects[0]
+def test_satin(fixture: PSDImage) -> None:
+    effect = fixture[11].effects[1]
+    assert isinstance(effect, effects.Satin)
     assert effect.angle == -60.0
     assert effect.anti_aliased is True
     assert effect.blend_mode == Enum.Multiply
