@@ -1,9 +1,7 @@
-from __future__ import absolute_import, unicode_literals
-
+from typing import Any, Type
 import os
 
 import pytest
-from IPython.display import display
 
 from psd_tools.psd.descriptor import (
     TYPES,
@@ -24,23 +22,22 @@ DESCRIPTOR_DATA = ["0.dat", "1.dat"]
 
 
 @pytest.mark.parametrize("cls", [TYPES[key] for key in TYPES])
-def test_empty_wr(cls):
+def test_empty_wr(cls: Type[Any]) -> None:
     check_write_read(cls())
 
 
 @pytest.mark.parametrize("filename", DESCRIPTOR_DATA)
-def test_descriptor_rw(filename):
+def test_descriptor_rw(filename: str) -> None:
     filepath = os.path.join(TEST_ROOT, "descriptors", filename)
     with open(filepath, "rb") as f:
         check_read_write(Descriptor, f.read())
 
 
 @pytest.mark.parametrize("filename", DESCRIPTOR_DATA)
-def test_descriptor_display(filename):
+def test_descriptor_display(filename: str) -> None:
     filepath = os.path.join(TEST_ROOT, "descriptors", filename)
     with open(filepath, "rb") as f:
-        value = Descriptor.frombytes(f.read())
-    display(value)
+        Descriptor.frombytes(f.read())
 
 
 @pytest.mark.parametrize(
@@ -52,7 +49,7 @@ def test_descriptor_display(filename):
         )
     ],
 )
-def test_reference_rw(fixture):
+def test_reference_rw(fixture: bytes) -> None:
     check_read_write(Reference, fixture)
 
 
@@ -66,7 +63,7 @@ def test_reference_rw(fixture):
         (Integer, 1),
     ],
 )
-def test_value_elements(kls, value):
+def test_value_elements(kls: Type[Any], value: Any) -> None:
     fixture = kls(value)
     assert fixture == value
 
@@ -78,7 +75,7 @@ def test_value_elements(kls, value):
         (Unit.Points, 0.0),
     ],
 )
-def test_unit_float(unit, value):
+def test_unit_float(unit: Unit, value: float) -> None:
     fixture = UnitFloat(unit=unit, value=value)
     assert fixture == value
     assert fixture + 1.0
@@ -92,5 +89,5 @@ def test_unit_float(unit, value):
         b"#Pxl\x00\x00\x00\x00\x00\x00\x00\x00",
     ],
 )
-def test_unit_float_enum(fixture):
+def test_unit_float_enum(fixture: bytes) -> None:
     UnitFloat.frombytes(fixture)
