@@ -1,5 +1,4 @@
-from __future__ import absolute_import, unicode_literals
-
+from typing import Any, Type
 import logging
 
 import pytest
@@ -29,7 +28,7 @@ logger = logging.getLogger(__name__)
         SolidFillInfo,
     ],
 )
-def test_effects_layer_empty_wr(kls):
+def test_effects_layer_empty_wr(kls: Type[Any]) -> None:
     check_write_read(kls())
 
 
@@ -42,5 +41,5 @@ def test_effects_layer_empty_wr(kls):
         ),
     ],
 )
-def test_solid_fill_info(fixture):
+def test_solid_fill_info(fixture: bytes) -> None:
     check_read_write(SolidFillInfo, fixture)
