@@ -1,9 +1,7 @@
-from __future__ import absolute_import, unicode_literals
-
+from typing import Any, Type
 import os
 
 import pytest
-from IPython.display import display
 
 from psd_tools.psd import PSD
 from psd_tools.constants import Resource, ColorMode
@@ -17,18 +15,17 @@ from psd_tools.psd.image_resources import (
 from ..utils import TEST_ROOT, check_read_write, check_write_read
 
 
-def test_image_resources_from_to():
+def test_image_resources_from_to() -> None:
     check_read_write(ImageResources, b"\x00\x00\x00\x00")
 
 
-def test_image_resources_exception():
+def test_image_resources_exception() -> None:
     with pytest.raises(IOError):
         ImageResources.frombytes(b"\x00\x00\x00\x01")
 
 
-def test_image_resources_dict():
+def test_image_resources_dict() -> None:
     image_resources = ImageResources.new()
-    display(image_resources)
     assert image_resources.get_data(Resource.VERSION_INFO)
     assert image_resources.get_data(Resource.OBSOLETE1) is None
     assert len([1 for key in image_resources if key == Resource.VERSION_INFO]) == 1
@@ -41,11 +38,11 @@ def test_image_resources_dict():
         (ImageResource(name="foo", data=b"\x01\x04\x02"),),
     ],
 )
-def test_image_resource_from_to(fixture):
+def test_image_resource_from_to(fixture: ImageResource) -> None:
     check_write_read(fixture)
 
 
-def test_image_resource_exception():
+def test_image_resource_exception() -> None:
     with pytest.raises(IOError):
         ImageResource.frombytes(b"\x00\x00\x00\x01")
 
@@ -56,20 +53,19 @@ def test_image_resource_exception():
         (Slices, "slices_0.dat"),
     ],
 )
-def test_image_resource_rw(kls, filename):
+def test_image_resource_rw(kls: Type[Any], filename: str) -> None:
     filepath = os.path.join(TEST_ROOT, "image_resources", filename)
     with open(filepath, "rb") as f:
         fixture = f.read()
     check_read_write(kls, fixture)
 
 
-def test_display_info():
+def test_display_info() -> None:
     filepath = os.path.join(TEST_ROOT, "psd_files", "cmyk-spot.psd")
     with open(filepath, "rb") as f:
         psd = PSD.read(f)
     assert psd.header.color_mode == ColorMode.CMYK
     info = psd.image_resources[Resource.DISPLAY_INFO].data
-    print(display(info))
     assert len(info.alpha_channels) == 3
     expected_colors = [
         # RGB in uint16, with an extra zero because the color space can be CMYK
@@ -83,14 +79,13 @@ def test_display_info():
         assert [c.c1, c.c2, c.c3, c.c4] == expected_colors[i]
 
 
-def test_display_info_channel_type():
+def test_display_info_channel_type() -> None:
     filepath = os.path.join(TEST_ROOT, "psd_files", "cmyk-alpha-spot.psd")
     with open(filepath, "rb") as f:
         psd = PSD.read(f)
 
     assert psd.header.color_mode == ColorMode.CMYK
     info = psd.image_resources[Resource.DISPLAY_INFO].data
-    print(display(info))
     assert len(info.alpha_channels) == 2
     assert info.alpha_channels[0].mode == AlphaChannelMode.SPOT
     assert info.alpha_channels[1].mode == AlphaChannelMode.INVERTED_ALPHA
