@@ -1,8 +1,6 @@
-from __future__ import absolute_import, unicode_literals
-
 import pytest
 
-from psd_tools.psd.vector import Path
+from psd_tools.psd.vector import Path, ClosedPath, OpenPath
 
 from ..utils import check_read_write
 
@@ -22,5 +20,12 @@ from ..utils import check_read_write
         b"\x00B\x8f\\\x00\x00"
     ],
 )
-def test_path_rw(fixture):
+def test_path_rw(fixture: bytes) -> None:
     check_read_write(Path, fixture)
+
+
+def test_subpath_repr() -> None:
+    closedpath = ClosedPath()
+    assert repr(closedpath) == "ClosedPath(index=0, operation=1)"
+    openpath = OpenPath()
+    assert repr(openpath) == "OpenPath(index=0, operation=1)"
