@@ -1,11 +1,13 @@
-from __future__ import absolute_import, unicode_literals
-
 import fnmatch
 import logging
 import os
 import tempfile
+from typing import Any, Generator, List, Type, TypeVar
 
-from psd_tools.utils import trimmed_repr
+from psd_tools.psd.base import BaseElement
+from psd_tools.psd.bin_utils import trimmed_repr
+
+T = TypeVar("T", bound=BaseElement)
 
 logging.basicConfig(level=logging.DEBUG)
 
@@ -33,21 +35,23 @@ OTHER_FILES = {
 TEST_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
 
 
-def find_files(pattern="*.ps*", root=TEST_ROOT):
+def find_files(
+    pattern: str = "*.ps*", root: str = TEST_ROOT
+) -> Generator[str, None, None]:
     for r, _, filenames in os.walk(root):
         for filename in fnmatch.filter(filenames, pattern):
             yield os.path.join(r, filename)
 
 
-def full_name(filename):
+def full_name(filename: str) -> str:
     return os.path.join(TEST_ROOT, "psd_files", filename)
 
 
-def all_files():
+def all_files() -> List[str]:
     return [f for f in find_files() if f.find("third-party-psds") < 0]
 
 
-def check_write_read(element, *args, **kwargs):
+def check_write_read(element: T, *args: Any, **kwargs: Any) -> None:
     with tempfile.TemporaryFile() as f:
         element.write(f, *args, **kwargs)
         f.flush()
@@ -56,7 +60,7 @@ def check_write_read(element, *args, **kwargs):
     assert element == new_element, "%s vs %s" % (element, new_element)
 
 
-def check_read_write(cls, data, *args, **kwargs):
+def check_read_write(cls: Type[T], data: bytes, *args: Any, **kwargs: Any) -> None:
     element = cls.frombytes(data, *args, **kwargs)
     new_data = element.tobytes(*args, **kwargs)
     assert data == new_data, "%s vs %s" % (trimmed_repr(data), trimmed_repr(new_data))
